@@ -7,7 +7,6 @@ from collections import defaultdict, namedtuple
 from logging import getLogger
 import os
 from os.path import basename, dirname, isdir, join
-from subprocess import CalledProcessError
 import sys
 from traceback import format_exception_only
 import warnings
@@ -17,7 +16,7 @@ from .path_actions import (CompileMultiPycAction, CreateNonadminAction, CreatePr
                            CreatePythonEntryPointAction, LinkPathAction, MakeMenuAction,
                            RegisterEnvironmentLocationAction, RemoveLinkedPackageRecordAction,
                            RemoveMenuAction, UnlinkPathAction, UnregisterEnvironmentLocationAction,
-                           UpdateHistoryAction)
+                           UpdateHistoryAction, AggregateCompileMultiPycAction)
 from .prefix_data import PrefixData, get_python_version_for_prefix
 from .. import CondaError, CondaMultiError, conda_signal_handler
 from .._vendor.auxlib.collection import first
@@ -27,7 +26,7 @@ from ..base.constants import DEFAULTS_CHANNEL_NAME, PREFIX_MAGIC_FILE, SafetyChe
 from ..base.context import context
 from ..common.compat import ensure_text_type, iteritems, itervalues, odict, on_win, text_type
 from ..common.io import Spinner, dashlist, time_recorder
-from ..common.io import DummyExecutor, ThreadLimitedThreadPoolExecutor, as_completed
+from ..common.io import DummyExecutor, ThreadLimitedThreadPoolExecutor
 from ..common.path import (explode_directories, get_all_directories, get_major_minor_version,
                            get_python_site_packages_short_path)
 from ..common.signals import signal_handler
@@ -66,10 +65,6 @@ def make_unlink_actions(transaction_context, target_prefix, prefix_record):
                                                  target_prefix, trgt)
                                 for trgt in prefix_record.files)
 
-    remove_menu_actions = RemoveMenuAction.create_actions(transaction_context,
-                                                          prefix_record,
-                                                          target_prefix)
-
     try:
         extracted_package_dir = basename(prefix_record.extracted_package_dir)
     except AttributeError:
@@ -96,7 +91,6 @@ def make_unlink_actions(transaction_context, target_prefix, prefix_record):
     # )
 
     return tuple(concatv(
-        remove_menu_actions,
         unlink_path_actions,
         directory_remove_actions,
         # unregister_private_package_actions,
@@ -125,12 +119,15 @@ PrefixSetup = namedtuple('PrefixSetup', (
 ))
 
 PrefixActionGroup = namedtuple('PrefixActionGroup', (
+    'remove_menu_action_groups',
     'unlink_action_groups',
     'unregister_action_groups',
     'link_action_groups',
     'register_action_groups',
     'compile_action_groups',
-    'prefix_record_groups'
+    'make_menu_action_groups',
+    'entry_point_action_groups',
+    'prefix_record_groups',
 ))
 
 # each PrefixGroup item is a sequence of ActionGroups
@@ -174,7 +171,12 @@ class UnlinkLinkTransaction(object):
         self._pfe = None
         self._prepared = False
         self._verified = False
-        self.executor = DummyExecutor() if context.debug else ThreadLimitedThreadPoolExecutor()
+        # this can be CPU-bound.  Use ProcessPoolExecutor.
+        self.verify_executor = (DummyExecutor() if context.debug or context.verify_threads == 1
+                                else ThreadLimitedThreadPoolExecutor(context.verify_threads))
+        # this is more I/O bound.  Use ThreadPoolExecutor.
+        self.execute_executor = (DummyExecutor() if context.debug or context.execute_threads == 1
+                                 else ThreadLimitedThreadPoolExecutor(context.execute_threads))
 
     @property
     def nothing_to_do(self):
@@ -297,60 +299,95 @@ class UnlinkLinkTransaction(object):
 
         transaction_context['temp_dir'] = join(target_prefix, '.condatmp')
 
-        unlink_action_groups = tuple(ActionGroup(
-            'unlink',
-            prefix_rec,
-            make_unlink_actions(transaction_context, target_prefix, prefix_rec),
-            target_prefix,
-        ) for prefix_rec in prefix_recs_to_unlink)
+        remove_menu_action_groups = []
+        unlink_action_groups = []
+        for prefix_rec in prefix_recs_to_unlink:
+            unlink_action_groups.append(ActionGroup(
+                'unlink',
+                prefix_rec,
+                make_unlink_actions(transaction_context, target_prefix, prefix_rec),
+                target_prefix))
+
+            remove_menu_action_groups.append(ActionGroup(
+                'remove_menus',
+                prefix_rec,
+                RemoveMenuAction.create_actions(
+                    transaction_context, prefix_rec, target_prefix),
+                target_prefix))
 
         if unlink_action_groups:
             axns = UnregisterEnvironmentLocationAction(transaction_context, target_prefix),
-            unregister_action_groups = ActionGroup('unregister', None, axns, target_prefix),
+            unregister_action_groups = [ActionGroup('unregister', None, axns, target_prefix)]
         else:
             unregister_action_groups = ()
 
         matchspecs_for_link_dists = match_specs_to_dists(packages_info_to_link, update_specs)
-        link_action_groups = tuple(
-            ActionGroup('link', pkg_info, cls._make_link_actions(transaction_context, pkg_info,
-                                                                 target_prefix, lt, spec),
-                        target_prefix)
-            for pkg_info, lt, spec in zip(packages_info_to_link, link_types,
-                                          matchspecs_for_link_dists)
-        )
-
-        compile_action_groups = tuple(
-            ActionGroup('compile', pkg_info, cls._make_compile_actions(
-                transaction_context, pkg_info, target_prefix, lt, spec, link_action_groups),
-                        target_prefix)
-            for pkg_info, lt, spec in zip(packages_info_to_link, link_types,
-                                          matchspecs_for_link_dists)
-        )
-
+        link_action_groups = []
+        entry_point_action_groups = []
+        compile_action_groups = []
+        make_menu_action_groups = []
         record_axns = []
         for pkg_info, lt, spec in zip(packages_info_to_link, link_types,
                                       matchspecs_for_link_dists):
-            link_ag = next(ag for ag in link_action_groups if ag.pkg_data == pkg_info)
-            compile_ag = next(ag for ag in compile_action_groups if ag.pkg_data == pkg_info)
-            all_link_path_actions = concatv(link_ag.actions, compile_ag.actions)
+            link_ag = ActionGroup(
+                'link',
+                pkg_info,
+                cls._make_link_actions(transaction_context, pkg_info,
+                                       target_prefix, lt, spec),
+                target_prefix)
+            link_action_groups.append(link_ag)
+
+            entry_point_ag = ActionGroup(
+                'entry_point',
+                pkg_info,
+                cls._make_entry_point_actions(
+                    transaction_context, pkg_info, target_prefix,
+                    lt, spec, link_action_groups),
+                target_prefix)
+            entry_point_action_groups.append(entry_point_ag)
+
+            compile_ag = ActionGroup(
+                'compile',
+                pkg_info,
+                cls._make_compile_actions(
+                    transaction_context, pkg_info, target_prefix,
+                    lt, spec, link_action_groups),
+                target_prefix)
+            compile_action_groups.append(compile_ag)
+
+            make_menu_ag = ActionGroup(
+                'make_menus',
+                pkg_info,
+                MakeMenuAction.create_actions(
+                    transaction_context, pkg_info, target_prefix, lt),
+                target_prefix)
+            make_menu_action_groups.append(make_menu_ag)
+
+            all_link_path_actions = concatv(link_ag.actions,
+                                            compile_ag.actions,
+                                            entry_point_ag.actions,
+                                            make_menu_ag.actions)
             record_axns.extend(CreatePrefixRecordAction.create_actions(
                 transaction_context, pkg_info, target_prefix, lt, spec, all_link_path_actions))
+
         prefix_record_groups = [ActionGroup('record', None, record_axns, target_prefix)]
 
         history_actions = UpdateHistoryAction.create_actions(
             transaction_context, target_prefix, remove_specs, update_specs,
         )
         register_actions = RegisterEnvironmentLocationAction(transaction_context, target_prefix),
-
-        register_action_groups = ActionGroup('register', None,
-                                             register_actions + history_actions,
-                                             target_prefix),
+        register_action_groups = [ActionGroup('register', None,
+                                              register_actions + history_actions,
+                                              target_prefix)]
         return PrefixActionGroup(
+            remove_menu_action_groups,
             unlink_action_groups,
             unregister_action_groups,
             link_action_groups,
-            compile_action_groups,
             register_action_groups,
+            compile_action_groups,
+            make_menu_action_groups,
+            entry_point_action_groups,
             prefix_record_groups,
         )
 
@@ -360,7 +397,7 @@ class UnlinkLinkTransaction(object):
                              for action_groups in prefix_action_group
                              for axngroup in action_groups)
 
-        # run all per-action verify methods
+        # run all per-action (per-package) verify methods
         #   one of the more important of these checks is to verify that a file listed in
         #   the packages manifest (i.e. info/files) is actually contained within the package
         error_results = []
@@ -375,7 +412,7 @@ class UnlinkLinkTransaction(object):
         return error_results
 
     @staticmethod
-    def _verify_prefix_level(target_prefix, prefix_action_group):
+    def _verify_prefix_level(target_prefix_AND_prefix_action_group_tuple):
         # further verification of the whole transaction
         # for each path we are creating in link_actions, we need to make sure
         #   1. each path either doesn't already exist in the prefix, or will be unlinked
@@ -385,6 +422,11 @@ class UnlinkLinkTransaction(object):
         #   5. make sure envs/catalog.json is writable; done with RegisterEnvironmentLocationAction
         # TODO: 3, 4
 
+        # this strange unpacking is to help the parallel execution work.  Unpacking
+        #    tuples in the map call could be done with a lambda, but that is then not picklable,
+        #    which precludes the use of ProcessPoolExecutor (but not ThreadPoolExecutor)
+        target_prefix, prefix_action_group = target_prefix_AND_prefix_action_group_tuple
+
         unlink_action_groups = prefix_action_group.unlink_action_groups
         prefix_record_groups = prefix_action_group.prefix_record_groups
 
@@ -499,14 +541,14 @@ class UnlinkLinkTransaction(object):
                                            or ())
             conda_linked_depends = conda_prec.depends
 
-        for conda_dependency in conda_linked_depends:
-            dep_name = MatchSpec(conda_dependency).name
-            if dep_name not in pkg_names_being_lnkd and (dep_name not in pkg_names_already_lnkd
-                                                         or dep_name in pkg_names_being_unlnkd):
-                # equivalent to not (dep_name in pkg_names_being_lnkd or
-                #  (dep_name in pkg_names_already_lnkd and dep_name not in pkg_names_being_unlnkd))
-                yield RemoveError("'%s' is a dependency of conda and cannot be removed from\n"
-                                  "conda's operating environment." % dep_name)
+        if conda_final_prefix in prefix_setups:
+            for conda_dependency in conda_linked_depends:
+                dep_name = MatchSpec(conda_dependency).name
+                if dep_name not in pkg_names_being_lnkd and (
+                        dep_name not in pkg_names_already_lnkd or
+                        dep_name in pkg_names_being_unlnkd):
+                    yield RemoveError("'%s' is a dependency of conda and cannot be removed from\n"
+                                      "conda's operating environment." % dep_name)
 
         # Verification 3. enforce disallowed_packages
         disallowed = tuple(MatchSpec(s) for s in context.disallowed_packages)
@@ -541,14 +583,15 @@ class UnlinkLinkTransaction(object):
             return transaction_exceptions
 
         exceptions = []
-        futures = list(self.executor.submit(UnlinkLinkTransaction._verify_individual_level, pg)
-                       for pg in itervalues(prefix_action_groups))
-        futures.extend(
-            self.executor.submit(UnlinkLinkTransaction._verify_prefix_level, target_prefix, pg)
-            for target_prefix, pg in iteritems(prefix_action_groups))
-        for future in as_completed(futures):
-            if future.result():
-                exceptions.extend(future.result())
+        for exc in self.verify_executor.map(UnlinkLinkTransaction._verify_individual_level,
+                                            itervalues(prefix_action_groups)):
+            if exc:
+                exceptions.extend(exc)
+        for exc in self.verify_executor.map(
+                UnlinkLinkTransaction._verify_prefix_level,
+                iteritems(prefix_action_groups)):
+            if exc:
+                exceptions.extend(exc)
         return exceptions
 
     def _execute(self, all_action_groups):
@@ -557,7 +600,13 @@ class UnlinkLinkTransaction(object):
         # link unlink_action_groups and register_action_groups
         link_actions = list(group for group in all_action_groups if group.type == "link")
         compile_actions = list(group for group in all_action_groups if group.type == "compile")
+        entry_point_actions = list(group for group in all_action_groups
+                                   if group.type == "entry_point")
         record_actions = list(group for group in all_action_groups if group.type == "record")
+        make_menu_actions = list(group for group in all_action_groups
+                                 if group.type == "make_menus")
+        remove_menu_actions = list(group for group in all_action_groups
+                                   if group.type == "remove_menus")
 
         with signal_handler(conda_signal_handler), time_recorder("unlink_link_execute"):
             exceptions = []
@@ -569,6 +618,12 @@ class UnlinkLinkTransaction(object):
                         (unlink_actions, "unregister", False),
                         (link_actions, "register", True)):
 
+                    if not install_side:
+                        # uninstalling menus must happen prior to unlinking, or else they might
+                        #   call something that isn't there anymore
+                        for axngroup in remove_menu_actions:
+                            UnlinkLinkTransaction._execute_actions(axngroup)
+
                     for axngroup in group:
                         is_unlink = axngroup.type == 'unlink'
                         target_prefix = axngroup.target_prefix
@@ -579,37 +634,38 @@ class UnlinkLinkTransaction(object):
                                    target_prefix)
 
                     # parallel block 1:
-                    futures = (
-                        self.executor.submit(UnlinkLinkTransaction._execute_actions, axngroup)
-                        for axngroup in group)
-                    for future in as_completed(futures):
-                        exc = future.result()
+                    for exc in self.execute_executor.map(UnlinkLinkTransaction._execute_actions,
+                                                         group):
                         if exc:
-                            log.debug('%r'.encode('utf-8'), exc.errors[0], exc_info=True)
                             exceptions.append(exc)
 
+                    # post link scripts may employ entry points.  Do them before post-link.
+                    if install_side:
+                        for axngroup in entry_point_actions:
+                            UnlinkLinkTransaction._execute_actions(axngroup)
+
                     # Run post-link or post-unlink scripts and registering AFTER link/unlink,
                     #    because they may depend on files in the prefix.  Additionally, run
                     #    them serially, just in case order matters (hopefully not)
                     for axngroup in group:
                         exc = UnlinkLinkTransaction._execute_post_link_actions(axngroup)
                         if exc:
-                            log.debug('%r'.encode('utf-8'), exc.errors[0], exc_info=True)
                             exceptions.append(exc)
 
                     # parallel block 2:
-                    futures = []
+                    composite_ag = []
                     if install_side:
-                        futures.extend(
-                            self.executor.submit(UnlinkLinkTransaction._execute_actions, axngroup)
-                            for axngroup in compile_actions)
-                        futures.extend(
-                            self.executor.submit(UnlinkLinkTransaction._execute_actions, axngroup)
-                            for axngroup in record_actions)
-                    for future in as_completed(futures):
-                        exc = future.result()
+                        composite_ag.extend(record_actions)
+                        # consolidate compile actions into one big'un for better efficiency
+                        individual_actions = [axn for ag in compile_actions for axn in ag.actions]
+                        if individual_actions:
+                            composite = AggregateCompileMultiPycAction(*individual_actions)
+                            composite_ag.append(ActionGroup('compile', None, [composite],
+                                                            composite.target_prefix))
+                    # functions return None unless there was an exception
+                    for exc in self.execute_executor.map(UnlinkLinkTransaction._execute_actions,
+                                                         composite_ag):
                         if exc:
-                            log.debug('%r'.encode('utf-8'), exc.errors[0], exc_info=True)
                             exceptions.append(exc)
 
                     # must do the register actions AFTER all link/unlink is done
@@ -618,10 +674,14 @@ class UnlinkLinkTransaction(object):
                     for axngroup in register_actions:
                         exc = UnlinkLinkTransaction._execute_actions(axngroup)
                         if exc:
-                            log.debug('%r'.encode('utf-8'), exc.errors[0], exc_info=True)
                             exceptions.append(exc)
                     if exceptions:
                         break
+                    if install_side:
+                        # uninstalling menus must happen prior to unlinking, or else they might
+                        #   call something that isn't there anymore
+                        for axngroup in make_menu_actions:
+                            UnlinkLinkTransaction._execute_actions(axngroup)
             if exceptions:
                 # might be good to show all errors, but right now we only show the first
                 e = exceptions[0]
@@ -630,11 +690,10 @@ class UnlinkLinkTransaction(object):
                 action, is_unlink = (None, axngroup.type == 'unlink')
                 prec = axngroup.pkg_data
 
-                log.error("An error occurred while %s package '%s'.\n"
-                          "%r\n"
-                          "Attempting to roll back.\n",
-                          'uninstalling' if is_unlink else 'installing',
-                          prec and prec.dist_str(), e.errors[0])
+                if prec:
+                    log.error("An error occurred while %s package '%s'." % (
+                            'uninstalling' if is_unlink else 'installing',
+                            prec.dist_str()))
 
                 # reverse all executed packages except the one that failed
                 rollback_excs = []
@@ -653,9 +712,6 @@ class UnlinkLinkTransaction(object):
                     rollback_excs,
                 )))
             else:
-
-                link_actions = tuple(
-                    group for group in all_action_groups if group.type in ("link", "register"))
                 for axngroup in all_action_groups:
                     for action in axngroup.actions:
                         action.cleanup()
@@ -663,7 +719,6 @@ class UnlinkLinkTransaction(object):
     @staticmethod
     def _execute_actions(axngroup):
         target_prefix = axngroup.target_prefix
-        action = None
         prec = axngroup.pkg_data
 
         conda_meta_dir = join(target_prefix, 'conda-meta')
@@ -686,13 +741,8 @@ class UnlinkLinkTransaction(object):
                 action.execute()
         except Exception as e:  # this won't be a multi error
             # reverse this package
-            log.debug("Error in action %r", action, exc_info=True)
             reverse_excs = ()
             if context.rollback_enabled:
-                # log.error("An error occurred while %s package '%s'.\n"
-                #           "%r\n"
-                #           "Attempting to roll back.\n",
-                #           'uninstalling' if is_unlink else 'installing', prec.dist_str(), e)
                 reverse_excs = UnlinkLinkTransaction._reverse_actions(axngroup)
             return CondaMultiError(tuple(concatv(
                 (e,),
@@ -711,13 +761,8 @@ class UnlinkLinkTransaction(object):
                            activate=True)
             except Exception as e:  # this won't be a multi error
                 # reverse this package
-                log.debug("Error in post-link", exc_info=True)
                 reverse_excs = ()
                 if context.rollback_enabled:
-                    log.error("An error occurred while %s package '%s'.\n"
-                              "%r\n"
-                              "Attempting to roll back.\n",
-                              'uninstalling' if is_unlink else 'installing', prec.dist_str(), e)
                     reverse_excs = UnlinkLinkTransaction._reverse_actions(axngroup)
                 return CondaMultiError(tuple(concatv(
                     (e,),
@@ -792,7 +837,6 @@ class UnlinkLinkTransaction(object):
             *required_quad, file_link_actions=file_link_actions
         )
         create_nonadmin_actions = CreateNonadminAction.create_actions(*required_quad)
-        create_menu_actions = MakeMenuAction.create_actions(*required_quad)
 
         # if requested_spec:
         #     application_entry_point_actions = CreateApplicationEntryPointAction.create_actions(
@@ -821,23 +865,23 @@ class UnlinkLinkTransaction(object):
             create_directory_actions,
             file_link_actions,
             create_nonadmin_actions,
-            create_menu_actions,
             # application_entry_point_actions,
             # register_private_env_actions,
         ))
 
+    @staticmethod
+    def _make_entry_point_actions(transaction_context, package_info, target_prefix,
+                                  requested_link_type, requested_spec, link_action_groups):
+        required_quad = transaction_context, package_info, target_prefix, requested_link_type
+        return CreatePythonEntryPointAction.create_actions(*required_quad)
+
     @staticmethod
     def _make_compile_actions(transaction_context, package_info, target_prefix,
                               requested_link_type, requested_spec, link_action_groups):
         required_quad = transaction_context, package_info, target_prefix, requested_link_type
-        python_entry_point_actions = CreatePythonEntryPointAction.create_actions(*required_quad)
         link_action_group = next(ag for ag in link_action_groups if ag.pkg_data == package_info)
-        compile_pyc_actions = CompileMultiPycAction.create_actions(
-            *required_quad, file_link_actions=link_action_group.actions)
-        return tuple(concatv(
-            python_entry_point_actions,
-            compile_pyc_actions,
-        ))
+        return CompileMultiPycAction.create_actions(*required_quad,
+                                                    file_link_actions=link_action_group.actions)
 
     def _make_legacy_action_groups(self):
         # this code reverts json output for plan back to previous behavior
@@ -1126,34 +1170,37 @@ def run_script(prefix, prec, action='post-link', env_prefix=None, activate=False
     env['PKG_BUILDNUM'] = prec.build_number
     env['PATH'] = os.pathsep.join((dirname(path), env.get('PATH', '')))
 
+    log.debug("for %s at %s, executing script: $ %s",
+              prec.dist_str(), env['PREFIX'], ' '.join(command_args))
     try:
-        log.debug("for %s at %s, executing script: $ %s",
-                  prec.dist_str(), env['PREFIX'], ' '.join(command_args))
-
-        subprocess_call(command_args, env=env, path=dirname(path))
-    except CalledProcessError:
-        m = messages(prefix)
-        if action in ('pre-link', 'post-link'):
-            if 'openssl' in prec.dist_str():
-                # this is a hack for conda-build string parsing in the conda_build/build.py
-                #   create_env function
-                message = "%s failed for: %s" % (action, prec)
+        response = subprocess_call(command_args, env=env, path=dirname(path), raise_on_error=False)
+        if response.rc != 0:
+            m = messages(prefix)
+            if action in ('pre-link', 'post-link'):
+                if 'openssl' in prec.dist_str():
+                    # this is a hack for conda-build string parsing in the conda_build/build.py
+                    #   create_env function
+                    message = "%s failed for: %s" % (action, prec)
+                else:
+                    message = dals("""
+                    %s script failed for package %s
+                    location of failed script: %s
+                    ==> script messages <==
+                    %s
+                    ==> script output <==
+                    stdout: %s
+                    stderr: %s
+                    return code: %s
+                    """) % (action, prec.dist_str(), path, m or "<None>",
+                            response.stdout, response.stderr, response.rc)
+                raise LinkError(message)
             else:
-                message = dals("""
-                %s script failed for package %s
-                running your command again with `-v` will provide additional information
-                location of failed script: %s
-                ==> script messages <==
-                %s
-                """) % (action, prec.dist_str(), path, m or "<None>")
-            raise LinkError(message)
+                log.warn("%s script failed for package %s\n"
+                         "consider notifying the package maintainer", action, prec.dist_str())
+                return False
         else:
-            log.warn("%s script failed for package %s\n"
-                     "consider notifying the package maintainer", action, prec.dist_str())
-            return False
-    else:
-        messages(prefix)
-        return True
+            messages(prefix)
+            return True
     finally:
         if script_caller is not None:
             if 'CONDA_TEST_SAVE_TEMPS' not in os.environ:
