@@ -81,7 +81,8 @@ class Solver(object):
         self._pool_cache = {}
 
     def solve_for_transaction(self, update_modifier=NULL, deps_modifier=NULL, prune=NULL,
-                              ignore_pinned=NULL, force_remove=NULL, force_reinstall=NULL):
+                              ignore_pinned=NULL, force_remove=NULL, force_reinstall=NULL,
+                              should_retry_solve=False):
         """Gives an UnlinkLinkTransaction instance that can be used to execute the solution
         on an environment.
 
@@ -96,6 +97,8 @@ class Solver(object):
                 See :meth:`solve_final_state`.
             force_reinstall (bool):
                 See :meth:`solve_for_diff`.
+            should_retry_solve (bool):
+                See :meth:`solve_final_state`.
 
         Returns:
             UnlinkLinkTransaction:
@@ -109,7 +112,8 @@ class Solver(object):
         else:
             unlink_precs, link_precs = self.solve_for_diff(update_modifier, deps_modifier,
                                                            prune, ignore_pinned,
-                                                           force_remove, force_reinstall)
+                                                           force_remove, force_reinstall,
+                                                           should_retry_solve)
             stp = PrefixSetup(self.prefix, unlink_precs, link_precs,
                               self.specs_to_remove, self.specs_to_add)
             # TODO: Only explicitly requested remove and update specs are being included in
@@ -119,7 +123,8 @@ class Solver(object):
             return UnlinkLinkTransaction(stp)
 
     def solve_for_diff(self, update_modifier=NULL, deps_modifier=NULL, prune=NULL,
-                       ignore_pinned=NULL, force_remove=NULL, force_reinstall=NULL):
+                       ignore_pinned=NULL, force_remove=NULL, force_reinstall=NULL,
+                       should_retry_solve=False):
         """Gives the package references to remove from an environment, followed by
         the package references to add to an environment.
 
@@ -137,6 +142,8 @@ class Solver(object):
                     instructs the solver to remove the package and spec from the environment,
                     and then add it back--possibly with the exact package instance modified,
                     depending on the spec exactness.
+            should_retry_solve (bool):
+                See :meth:`solve_final_state`.
 
         Returns:
             Tuple[PackageRef], Tuple[PackageRef]:
@@ -147,7 +154,7 @@ class Solver(object):
 
         """
         final_precs = self.solve_final_state(update_modifier, deps_modifier, prune, ignore_pinned,
-                                             force_remove)
+                                             force_remove, should_retry_solve)
         unlink_precs, link_precs = diff_for_unlink_link_precs(
             self.prefix, final_precs, self.specs_to_add, force_reinstall
         )
@@ -161,7 +168,7 @@ class Solver(object):
         return unlink_precs, link_precs
 
     def solve_final_state(self, update_modifier=NULL, deps_modifier=NULL, prune=NULL,
-                          ignore_pinned=NULL, force_remove=NULL):
+                          ignore_pinned=NULL, force_remove=NULL, should_retry_solve=False):
         """Gives the final, solved state of the environment.
 
         Args:
@@ -188,6 +195,9 @@ class Solver(object):
                 for the prefix.
             force_remove (bool):
                 Forces removal of a package without removing packages that depend on it.
+            should_retry_solve (bool):
+                Indicates whether this solve will be retried. This allows us to control
+                whether to call find_conflicts (slow) in ssc.r.solve
 
         Returns:
             Tuple[PackageRef]:
@@ -215,13 +225,15 @@ class Solver(object):
 
         if not retrying:
             ssc = SolverStateContainer(
-                self.prefix, update_modifier, deps_modifier, prune, ignore_pinned, force_remove
+                self.prefix, update_modifier, deps_modifier, prune, ignore_pinned, force_remove,
+                should_retry_solve,
             )
             self.ssc = ssc
         else:
             ssc = self.ssc
             ssc.update_modifier = update_modifier
             ssc.deps_modifier = deps_modifier
+            ssc.should_retry_solve = should_retry_solve
 
         # force_remove is a special case where we return early
         if self.specs_to_remove and force_remove:
@@ -248,8 +260,14 @@ class Solver(object):
                          context.json):
                 ssc = self._collect_all_metadata(ssc)
 
-        fail_message = ("failed\n" if self._repodata_fn == REPODATA_FN else "failed with %s, "
-                        "will retry with next repodata source.\n" % self._repodata_fn)
+        if should_retry_solve and update_modifier == UpdateModifier.FREEZE_INSTALLED:
+            fail_message = "failed with initial frozen solve. Retrying with flexible solve.\n"
+        elif self._repodata_fn != REPODATA_FN:
+            fail_message = ("failed with repodata from %s, will retry with next repodata"
+                            " source.\n" % self._repodata_fn)
+        else:
+            fail_message = "failed\n"
+
         with Spinner("Solving environment", not context.verbosity and not context.quiet,
                      context.json, fail_message=fail_message):
             ssc = self._remove_specs(ssc)
@@ -274,7 +292,11 @@ class Solver(object):
                 orphan_precs = (set(solution_precs)
                                 - set(ssc.solution_precs)
                                 - set(ssc.add_back_map))
-                ssc.solution_precs.extend([_ for _ in orphan_precs if _.name not in ssc.specs_map])
+                solution_prec_names = [_.name for _ in ssc.solution_precs]
+                ssc.solution_precs.extend(
+                    [_ for _ in orphan_precs
+                     if _.name not in ssc.specs_map and _.name not in solution_prec_names])
+
             ssc = self._post_sat_handling(ssc)
 
         time_recorder.log_totals()
@@ -359,7 +381,10 @@ class Solver(object):
         # add in historically-requested specs
         ssc.specs_map.update(ssc.specs_from_history_map)
 
-        for pkg_name in ('anaconda', 'conda', 'conda-build',
+        # these are things that we want to keep even if they're not explicitly specified.  This
+        #     is to compensate for older installers not recording these appropriately for them
+        #     to be preserved.
+        for pkg_name in ('anaconda', 'conda', 'conda-build', 'python.app',
                          'console_shortcut', 'powershell_shortcut'):
             if pkg_name not in ssc.specs_map and ssc.prefix_data.get(pkg_name, None):
                 ssc.specs_map[pkg_name] = MatchSpec(pkg_name)
@@ -492,7 +517,8 @@ class Solver(object):
                     has_update = True
                     break
         # let conda determine the latest version by just adding a name spec
-        return MatchSpec(spec.name) if has_update else None
+        return (MatchSpec(spec.name, version=prec.version, build_number=prec.build_number)
+                if has_update else spec)
 
     def _should_freeze(self, ssc, target_prec, conflict_specs, explicit_pool, installed_pool):
         # never, ever freeze anything if we have no history.
@@ -506,20 +532,10 @@ class Solver(object):
         pkg_name = target_prec.name
         no_conflict = (pkg_name not in conflict_specs and
                        (pkg_name not in explicit_pool or
-                        target_prec in explicit_pool[pkg_name]) and
-                       self._compare_pools(ssc, explicit_pool, target_prec.to_match_spec()))
+                        target_prec in explicit_pool[pkg_name]))
 
         return no_conflict
 
-    def _compare_pools(self, ssc, explicit_pool, ms):
-        other_pool = ssc.r._get_package_pool((ms, ))
-        match = True
-        for k in set(other_pool.keys()) & set(explicit_pool.keys()):
-            if not bool(other_pool[k] & explicit_pool[k]):
-                match = False
-                break
-        return match
-
     def _add_specs(self, ssc):
         # For the remaining specs in specs_map, add target to each spec. `target` is a reference
         # to the package currently existing in the environment. Setting target instructs the
@@ -537,8 +553,9 @@ class Solver(object):
         #    specs being added.
         explicit_pool = ssc.r._get_package_pool(self.specs_to_add)
 
-        conflict_specs = ssc.r.get_conflicting_specs(list(concatv(
-            self.specs_to_add, (_.to_match_spec() for _ in ssc.prefix_data.iter_records())))) or []
+        conflict_specs = ssc.r.get_conflicting_specs(tuple(concatv(
+            (_.to_match_spec() for _ in ssc.prefix_data.iter_records()))), self.specs_to_add
+        ) or tuple()
         conflict_specs = set(_.name for _ in conflict_specs)
 
         for pkg_name, spec in iteritems(ssc.specs_map):
@@ -559,7 +576,7 @@ class Solver(object):
                 if target_prec.is_unmanageable:
                     ssc.specs_map[pkg_name] = target_prec.to_match_spec()
                 elif MatchSpec(pkg_name) in context.aggressive_update_packages:
-                    ssc.specs_map[pkg_name] = pkg_name
+                    ssc.specs_map[pkg_name] = MatchSpec(pkg_name)
                 elif self._should_freeze(ssc, target_prec, conflict_specs, explicit_pool,
                                          installed_pool):
                     ssc.specs_map[pkg_name] = target_prec.to_match_spec()
@@ -582,21 +599,20 @@ class Solver(object):
                     log.warn("pinned spec %s conflicts with explicit specs.  "
                              "Overriding pinned spec.", s)
 
+        # we want to freeze any packages in the env that are not conflicts, so that the
+        #     solve goes faster.  This is kind of like an iterative solve, except rather
+        #     than just providing a starting place, we are preventing some solutions.
+        #     A true iterative solve would probably be better in terms of reaching the
+        #     optimal output all the time.  It would probably also get rid of the need
+        #     to retry with an unfrozen (UPDATE_SPECS) solve.
         if ssc.update_modifier == UpdateModifier.FREEZE_INSTALLED:
-            for prec in ssc.prefix_data.iter_records():
-                if prec.name not in ssc.specs_map:
-                    if (prec.name not in conflict_specs and
-                            (prec.name not in explicit_pool or
-                             prec in explicit_pool[prec.name]) and
-                            # because it's not just immediate deps, but also
-                            # upstream things that matter, we must ensure
-                            # overlap for dependencies of things that have
-                            # otherwise passed our tests
-                            self._compare_pools(ssc, explicit_pool, prec.to_match_spec())):
-                        ssc.specs_map[prec.name] = prec.to_match_spec()
-                    else:
-                        ssc.specs_map[prec.name] = MatchSpec(
-                            prec.name, target=prec.to_match_spec(), optional=True)
+            precs = [_ for _ in ssc.prefix_data.iter_records() if _.name not in ssc.specs_map]
+            for prec in precs:
+                if prec.name not in conflict_specs:
+                    ssc.specs_map[prec.name] = prec.to_match_spec()
+                else:
+                    ssc.specs_map[prec.name] = MatchSpec(
+                        prec.name, target=prec.to_match_spec(), optional=True)
         log.debug("specs_map with targets: %s", ssc.specs_map)
 
         # If we're in UPDATE_ALL mode, we need to drop all the constraints attached to specs,
@@ -627,21 +643,29 @@ class Solver(object):
                                       for prec in ssc.prefix_data.iter_records()
                                       )
 
+        # ensure that our self.specs_to_add are not being held back by packages in the env.
+        #    This factors in pins and also ignores specs from the history.  It is unfreezing only
+        #    for the indirect specs that otherwise conflict with update of the immediate request
         elif ssc.update_modifier == UpdateModifier.UPDATE_SPECS:
-            for spec in self.specs_to_add:
-                spec = MatchSpec(spec)
-                if (spec.name in pin_overrides and not ssc.ignore_pinned
-                        or spec.name in ssc.specs_from_history_map):
-                    # skip this spec, because it is constrained by pins
-                    continue
-                # the index is sorted, so the first record here gives us what we want.
-                latest_pkg = ssc.r.find_matches(spec)[0]
-                for ms in list(ssc.specs_map.values()):
-                    ms = MatchSpec(ms)
-                    spec_pool = ssc.r._get_package_pool([ms])
-                    if spec_pool.get(spec.name) and latest_pkg not in spec_pool[spec.name]:
-                        # neuter the spec due to a conflict
-                        ssc.specs_map[ms.name] = MatchSpec(ms.name)
+            skip = lambda x: ((x.name not in pin_overrides and
+                               any(x.name == _.name for _ in ssc.pinned_specs) and
+                               not ssc.ignore_pinned) or
+                              x.name in ssc.specs_from_history_map)
+
+            specs_to_add = tuple(self._package_has_updates(ssc, _, installed_pool)
+                                 for _ in self.specs_to_add if not skip(_))
+            # the index is sorted, so the first record here gives us what we want.
+            conflicts = ssc.r.get_conflicting_specs(tuple(MatchSpec(_)
+                                                          for _ in ssc.specs_map.values()),
+                                                    specs_to_add)
+            for conflict in conflicts:
+                # neuter the spec due to a conflict
+                if (conflict.name in ssc.specs_map and (
+                        # add optional because any pinned specs will include it
+                        MatchSpec(conflict, optional=True) not in ssc.pinned_specs or
+                        ssc.ignore_pinned) and
+                        conflict.name not in ssc.specs_from_history_map):
+                    ssc.specs_map[conflict.name] = MatchSpec(conflict.name)
 
         # As a business rule, we never want to update python beyond the current minor version,
         # unless that's requested explicitly by the user (which we actively discourage).
@@ -661,10 +685,10 @@ class Solver(object):
                     python_spec = MatchSpec(python_spec, version=pinned_version)
 
                 spec_set = (python_spec, ) + tuple(self.specs_to_add)
-                if ssc.r.get_conflicting_specs(spec_set):
+                if ssc.r.get_conflicting_specs(spec_set, self.specs_to_add):
                     if self._command != 'install' or (
                             self._repodata_fn == REPODATA_FN and
-                            ssc.update_modifier != UpdateModifier.FREEZE_INSTALLED):
+                            (not ssc.should_retry_solve or not freeze_installed)):
                         # raises a hopefully helpful error message
                         ssc.r.find_conflicts(spec_set)
                     else:
@@ -723,8 +747,8 @@ class Solver(object):
         # may not be the only unsatisfiable subset. We may have to call get_conflicting_specs()
         # several times, each time making modifications to loosen constraints.
 
-        conflicting_specs = set(ssc.r.get_conflicting_specs(final_environment_specs) or [])
-
+        conflicting_specs = set(ssc.r.get_conflicting_specs(tuple(final_environment_specs),
+                                                            self.specs_to_add) or [])
         while conflicting_specs:
             specs_modified = False
             if log.isEnabledFor(DEBUG):
@@ -765,7 +789,7 @@ class Solver(object):
                     ssc.specs_map[spec.name] = neutered_spec
             if specs_modified:
                 conflicting_specs = set(ssc.r.get_conflicting_specs(
-                    tuple(final_environment_specs)))
+                    tuple(final_environment_specs), self.specs_to_add))
             else:
                 # Let r.solve() use r.find_conflicts() to report conflict chains.
                 break
@@ -777,7 +801,8 @@ class Solver(object):
         ssc.solution_precs = ssc.r.solve(tuple(final_environment_specs),
                                          specs_to_add=self.specs_to_add,
                                          history_specs=ssc.specs_from_history_map,
-                                         repodata_fn=self._repodata_fn)
+                                         should_retry_solve=ssc.should_retry_solve
+                                         )
 
         # add back inconsistent packages to solution
         if ssc.add_back_map:
@@ -986,7 +1011,8 @@ class SolverStateContainer(object):
     # A mutable container with defined attributes to help keep method signatures clean
     # and also keep track of important state variables.
 
-    def __init__(self, prefix, update_modifier, deps_modifier, prune, ignore_pinned, force_remove):
+    def __init__(self, prefix, update_modifier, deps_modifier, prune, ignore_pinned, force_remove,
+                 should_retry_solve):
         # prefix, channels, subdirs, specs_to_add, specs_to_remove
         # self.prefix = prefix
         # self.channels = channels
@@ -1000,6 +1026,7 @@ class SolverStateContainer(object):
         self.prune = prune
         self.ignore_pinned = ignore_pinned
         self.force_remove = force_remove
+        self.should_retry_solve = should_retry_solve
 
         # Group 2. System state
         self.prefix = prefix
