@@ -314,11 +314,11 @@ def test_unsat_from_r1():
     with pytest.raises(UnsatisfiableError) as excinfo:
         r.install(['numpy 1.5*', 'scipy 0.12.0b1'])
     assert "numpy=1.5" in str(excinfo.value)
-    assert "scipy==0.12.0b1 -> numpy" in str(excinfo.value)
+    assert "scipy==0.12.0b1 -> numpy[version='1.6.*|1.7.*']" in str(excinfo.value)
     # numpy 1.5 does not have a python 3 package
     with pytest.raises(UnsatisfiableError) as excinfo:
         r.install(['numpy 1.5*', 'python 3*'])
-    assert "numpy=1.5 -> python[version='2.6.*,2.7.*']" in str(excinfo.value)
+    assert "numpy=1.5 -> python[version='2.6.*|2.7.*']" in str(excinfo.value)
     assert "python=3" in str(excinfo.value)
     with pytest.raises(UnsatisfiableError) as excinfo:
         r.install(['numpy 1.5*', 'numpy 1.6*'])
@@ -355,6 +355,56 @@ def test_unsat_simple():
     assert "b -> c[version='>=2,<3']" in str(excinfo.value)
 
 
+def test_unsat_shortest_chain_1():
+    index = (
+        simple_rec(name='a', depends=['d', 'c <1.3.0']),
+        simple_rec(name='b', depends=['c']),
+        simple_rec(name='c', version='1.3.6',),
+        simple_rec(name='c', version='1.2.8',),
+        simple_rec(name='d', depends=['c >=0.8.0']),
+    )
+    r = Resolve(OrderedDict((prec, prec) for prec in index))
+    with pytest.raises(UnsatisfiableError) as excinfo:
+        r.install(['c=1.3.6', 'a', 'b'])
+    assert "a -> c[version='<1.3.0']" in str(excinfo.value)
+    assert "b -> c" in str(excinfo.value)
+    assert "c=1.3.6" in str(excinfo.value)
+
+
+def test_unsat_shortest_chain_2():
+    index = (
+        simple_rec(name='a', depends=['d', 'c >=0.8.0']),
+        simple_rec(name='b', depends=['c']),
+        simple_rec(name='c', version='1.3.6',),
+        simple_rec(name='c', version='1.2.8',),
+        simple_rec(name='d', depends=['c <1.3.0']),
+    )
+    r = Resolve(OrderedDict((prec, prec) for prec in index))
+    with pytest.raises(UnsatisfiableError) as excinfo:
+        r.install(['c=1.3.6', 'a', 'b'])
+    assert "a -> d -> c[version='<1.3.0']" in str(excinfo.value)
+    assert "b -> c" in str(excinfo.value)
+    assert "c=1.3.6" in str(excinfo.value)
+
+
+def test_unsat_shortest_chain_3():
+    index = (
+        simple_rec(name='a', depends=['f', 'e']),
+        simple_rec(name='b', depends=['c']),
+        simple_rec(name='c', version='1.3.6',),
+        simple_rec(name='c', version='1.2.8',),
+        simple_rec(name='d', depends=['c >=0.8.0']),
+        simple_rec(name='e', depends=['c <1.3.0']),
+        simple_rec(name='f', depends=['d']),
+    )
+    r = Resolve(OrderedDict((prec, prec) for prec in index))
+    with pytest.raises(UnsatisfiableError) as excinfo:
+        r.install(['c=1.3.6', 'a', 'b'])
+    assert "a -> e -> c[version='<1.3.0']" in str(excinfo.value)
+    assert "b -> c" in str(excinfo.value)
+    assert "c=1.3.6" in str(excinfo.value)
+
+
 def test_unsat_chain():
     # a -> b -> c=1.x -> d=1.x
     # e      -> c=2.x -> d=2.x
@@ -407,13 +457,9 @@ def test_unsat_any_two_not_three():
     # a, b and c cannot be installed
     with pytest.raises(UnsatisfiableError) as excinfo:
         r.install(['a', 'b', 'c'])
-    assert "a -> d[version='>=1,<2,>=2,<3']" in str(excinfo.value)
-    assert "b -> d[version='>=1,<2,>=3,<4']" in str(excinfo.value)
-    assert "c -> d[version='>=2,<3,>=3,<4']" in str(excinfo.value)
-    # TODO would also like to see these
-    #assert "a -> d[version='>=1,<2']" in str(excinfo.value)
-    #assert "b -> d[version='>=1,<2']" in str(excinfo.value)
-    #assert "c -> d[version='>=2,<3']" in str(excinfo.value)
+    assert "a -> d[version='>=1,<2|>=2,<3']" in str(excinfo.value)
+    assert "b -> d[version='>=1,<2|>=3,<4']" in str(excinfo.value)
+    assert "c -> d[version='>=2,<3|>=3,<4']" in str(excinfo.value)
 
 
 def test_unsat_expand_single():
@@ -1251,6 +1297,7 @@ def test_channel_priority_1():
     assert installed2 == installed3
 
 
+@pytest.mark.integration
 def test_channel_priority_2():
     this_index = index.copy()
     index4, r4 = get_index_r_4()
